// leave at least 2 line with only a star on it below, or doc generation fails
/**
 *
 *
 * Placeholder for custom user javascript
 * mainly to be overridden in profile/static/custom/custom.js
 * This will always be an empty file in IPython
 *
 * User could add any javascript in the `profile/static/custom/custom.js` file.
 * It will be executed by the ipython notebook at load time.
 *
 * Same thing with `profile/static/custom/custom.css` to inject custom css into the notebook.
 *
 *
 * The object available at load time depend on the version of IPython in use.
 * there is no guaranties of API stability.
 *
 * The example below explain the principle, and might not be valid.
 *
 * Instances are created after the loading of this file and might need to be accessed using events:
 *     define([
 *        'base/js/namespace',
 *        'base/js/events'
 *     ], function(IPython, events) {
 *         events.on("app_initialized.NotebookApp", function () {
 *             IPython.keyboard_manager....
 *         });
 *     });
 *
 * __Example 1:__
 *
 * Create a custom button in toolbar that execute `%qtconsole` in kernel
 * and hence open a qtconsole attached to the same kernel as the current notebook
 *
 *    define([
 *        'base/js/namespace',
 *        'base/js/events'
 *    ], function(IPython, events) {
 *        events.on('app_initialized.NotebookApp', function(){
 *            IPython.toolbar.add_buttons_group([
 *                {
 *                    'label'   : 'run qtconsole',
 *                    'icon'    : 'icon-terminal', // select your icon from http://fortawesome.github.io/Font-Awesome/icons
 *                    'callback': function () {
 *                        IPython.notebook.kernel.execute('%qtconsole')
 *                    }
 *                }
 *                // add more button here if needed.
 *                ]);
 *        });
 *    });
 *
 * __Example 2:__
 *
 * At the completion of the dashboard loading, load an unofficial javascript extension
 * that is installed in profile/static/custom/
 *
 *    define([
 *        'base/js/events'
 *    ], function(events) {
 *        events.on('app_initialized.DashboardApp', function(){
 *            require(['custom/unofficial_extension.js'])
 *        });
 *    });
 *
 * __Example 3:__
 *
 *  Use `jQuery.getScript(url [, success(script, textStatus, jqXHR)] );`
 *  to load custom script into the notebook.
 *
 *    // to load the metadata ui extension example.
 *    $.getScript('/static/notebook/js/celltoolbarpresets/example.js');
 *    // or
 *    // to load the metadata ui extension to control slideshow mode / reveal js for nbconvert
 *    $.getScript('/static/notebook/js/celltoolbarpresets/slideshow.js');
 *
 *
 * @module IPython
 * @namespace IPython
 * @class customjs
 * @static
 */


var esriCDN =  location.protocol + "//js.arcgis.com/3.14amd/"
var proxyUrl = "/proxy/proxy.jsp" ;
    
require.config({
      // Define path mappings for modules
      paths: {
        // [1] Modules hosted on Esri CDN.
        "dojo":         esriCDN + "dojo",
        "dojox":        esriCDN + "dojox",
        "dijit":        esriCDN + "dijit",
        "esri":         esriCDN + "esri",
        "dgrid":        esriCDN + "dgrid",
        "xstyle":       esriCDN + "xstyle",
        "put-selector": esriCDN + "put-selector",
        
        // [2] Modules hosted locally.
        // "location" is specified as path relative to web server root.
        // "requirejs":  "/research/js/requirejs", - already loaded
        //"text":       "/static/custom/requirejs/text"
        "text":       "/nbextensions/arcgis/requirejs/text"
      },
      
      // Use RequireJS text plugin instead of dojo/text plugin.
      // Any module that requires dojo/text plugin will use RequireJS
      // text plugin instead.
      // http://requirejs.org/docs/api.html#config
      map: {
        "*": {
          "dojo/text": "text"
        }
      },
        config: {
        text: {
          
          useXhr: function(url) {
            // Allow cross domain XHR requests:
            // We will route them through a proxy in onXhr below.
            // https://github.com/requirejs/text/blob/master/text.js#L129
            
            return true;
          },
          
          // In IE 9, text plugin fails even before onXhr is called:
          // It fails right when calling xhr.open:
          // https://github.com/requirejs/text/blob/master/text.js#L267
          // - This is different from other browsers which appear to fail 
          // much later, allowing us a chance to append proxy below.
          // -- Probably because IE 9 does not support CORS as opposed to 
          // other modern browsers that have CORS support.
          
          // ESRI modification: let's take over xhr.open below
          openXhr: false,
          
          onXhr: function(xhr, url) {
            // Route cross domain XHR through a proxy if required
            var hasCors = (
              typeof XMLHttpRequest !== "undefined" 
              && ("withCredentials" in (new XMLHttpRequest()))
            );
            
            xhr.open(
              "GET", 
              hasCors ? url : (proxyUrl + "?" + url), 
              true
            );
          }
        }
      } 
    });

//require([
    //"widgets/js/widget", 
    //"widgets/js/manager",
    //"nbextensions/widgets/widgets/js/widget",
    //"nbextensions/widgets/widgets/js/manager",
require.undef('mapview');

define('mapview', [ 
     "jupyter-js-widgets",
     "esri/map",
     "esri/dijit/LayerSwipe",
     "esri/graphic",
     "esri/TimeExtent",
     "esri/geometry/Extent",
     "esri/InfoTemplate",
     "esri/toolbars/draw",
     "esri/layers/KMLLayer",
     "esri/layers/RasterFunction",
     "esri/layers/ArcGISImageServiceLayer", 
     "esri/layers/ImageServiceParameters", 
     "esri/geometry/Polyline",
     "esri/geometry/Polygon",
     "esri/geometry/Point",
     "esri/geometry/Multipoint",
     "esri/layers/FeatureLayer",
     "esri/renderers/smartMapping",
     "esri/symbols/SimpleFillSymbol",
     "esri/symbols/SimpleLineSymbol",
     "esri/renderers/HeatmapRenderer",
     "esri/symbols/PictureMarkerSymbol",
     "esri/geometry/webMercatorUtils",
     "esri/arcgis/utils",
     "dojo/domReady!"
   ], function( //widget, manager, 
		widgets,
		Map, LayerSwipe, Graphic, TimeExtent, Extent, InfoTemplate,
                Draw, KMLLayer, RasterFunction, ArcGISImageServiceLayer, 
                ImageServiceParameters,
                Polyline, Polygon, Point, Multipoint, FeatureLayer, 
                smartMapping, SimpleFillSymbol, SimpleLineSymbol, HeatmapRenderer,
                PictureMarkerSymbol, webMercatorUtils, arcgisUtils){
    
    var map, toolbar;
    var MapView = widgets.DOMWidgetView.extend({
        
        // Render the view.
        render: function(){ 
            $('head').append( $('<link rel="stylesheet" type="text/css" />').attr('href', '/nbextensions/arcgis/custom.css') );
            
            var that = this;
    
            var id = this.model.get('id')
            
            if ((id) && (id.trim()!='')) {
                arcgisUtils.createMap(id, this.$el[0]).then(function(response){
                    map = response.map;
                });
            } else {
                
                this.$el.append("<div id='"+this.model.get('_swipe_div')+"'>");
                
               map = new Map(this.$el[0], { 
                    basemap: this.model.get('basemap'),
                    center: this.model.get('center').reverse(),
                    zoom: this.model.get('zoom')
                });
            }
            
            map.on("load", on_load);
            
            function on_load() {     
                map.disableKeyboardNavigation(); // interferes with the Notebook keyboard shortcuts
                
                // create the draw toolbar, it activates only when mode=draw_*
                toolbar = new Draw(map);
                
                // hook up events
                toolbar.on("draw-end", onDrawEnd);
                //map.on("extent-change", onExtentChange);
                map.on("click", onMouseClick);
                
                var timeExtent = new TimeExtent();
                timeExtent.startTime = new Date("1/1/1989 UTC");
                timeExtent.endTime = new Date("1/1/1991 UTC");
                ///map.setTimeExtent(timeExtent);

            }
            
            // JS map events
            /*
            function onExtentChange(evt){
              var extent = evt.extent,
                  zoomed = evt.levelChange;
                that.extent_change(extent, zoomed);
            }
            */
            
            function onDrawEnd(evtObj){
              var geometry = evtObj.geometry;

              var graphic = map.graphics.add(new Graphic(geometry, new SimpleFillSymbol()));
              
              toolbar.deactivate();
              that.draw_end(geometry);
            }
            
            function onMouseClick(event) {
                console.log("User clicked at " +  event.screenPoint.x + ", " + event.screenPoint.y +
                            " on the screen. The map coordinate at this point is " + 
                            event.mapPoint.x + ", " + event.mapPoint.y
                
                );
                //console.log("MapCpoint:"+JSON.stringify(event.mapPoint));
                //var normalizedVal = webMercatorUtils.xyToLngLat(event.mapPoint.x, event.mapPoint.y);
                //console.log(normalizedVal);
                that.mouse_clicked(event.mapPoint);//normalizedVal[0], normalizedVal[1]);
            }
            
            // Model change events
            this.model.on('change:zoom', this.zoom_changed, this);
            this.model.on('change:mode', this.mode_changed, this);
            this.model.on('change:center', this.center_changed, this);  
            this.model.on('change:basemap', this.basemap_changed, this);  
            this.model.on('change:addlayer', this.layer_changed, this);  
            this.model.on('change:start_time', this.start_time_changed, this);  
            this.model.on('change:end_time', this.end_time_changed, this);  
            
        },
        
        // Incoming Events from the model
        zoom_changed: function() {
            map.setZoom(this.model.get('zoom'));
        },
        
        mode_changed: function() {
             
            if (this.model.get('mode') == "navigate") {
                console.log("***mode = navigate")
                toolbar.deactivate();
            } else if (this.model.get('mode') == "###clear_graphics") {
                map.graphics.clear();
            } else if (this.model.get('mode') == "###remove_layers") {
                map.removeAllLayers();
            } else if (this.model.get('mode').indexOf("{") > -1) {
                
                console.log("***mode=draw_geometry%%%" );
                var drawgraphic = JSON.parse(this.model.get('mode'));
                var gfx = new Graphic(drawgraphic);

                if (gfx.symbol == null) {
                    if (gfx.geometry.type === 'polyline') {
                        console.log("GEOM TYPE POLYLINE");
                        
                        gfx.symbol = new SimpleLineSymbol(SimpleLineSymbol.STYLE_SOLID, new dojo.Color([255,0,0,0.5]),3);
                    } else if (gfx.geometry.type === 'polygon') {
                        console.log("GEOM TYPE POLYGON");

                        gfx.symbol = new SimpleFillSymbol();
                    } else if (gfx.geometry.type === 'point') {
                        console.log("GEOM TYPE POINT");

                        gfx.symbol = new PictureMarkerSymbol('/nbextensions/arcgis/icons/marker.png', 32, 32);
                    } else if (gfx.geometry.type === 'multipoint') {
                        console.log("GEOM TYPE MULTIPOINT");

                        gfx.symbol = new PictureMarkerSymbol('/nbextensions/arcgis/icons/marker.png', 32, 32);
                    }
                }
                map.graphics.add(gfx); 
                
            } else {
                var shape = this.model.get('mode');
                console.log("***mode=draw_" + shape);
                toolbar.activate(shape);   
            }
        },
        
        layer_changed: function() {
             
            if (this.model.get('addlayer').indexOf("{") > -1) {
                
                console.log("***mode=addlayer%%%" );
                
                var newlayer = JSON.parse(this.model.get('addlayer'));
                console.log(newlayer);
                if (newlayer.type == "KMLLayer") {
                    console.log("KMLLayer " + newlayer.url);
                    var kml = new KMLLayer(newlayer.url); 
                    map.addLayer(kml);
                    kml.on("load", function() {
                      domStyle.set("loading", "display", "none");
                    });
                } 
                else if ((newlayer.type == "FeatureLayer") || (newlayer.type == "Feature Layer"))  
                {
                    console.log("FeatureLayer " + newlayer.url);
                    
                    var layer = new FeatureLayer(newlayer.url, {
                        "outFields":["*"]
                        }); 
                   
                    if (newlayer.opacity != null) {
                        layer.setOpacity(newlayer.opacity);
                    }
                    
                    if (newlayer.definition_expression != null) {
                        console.log("***DEF EXP");
                        console.log(newlayer.definition_expression);
                        layer.setDefinitionExpression(newlayer.definition_expression);
                    }
                    
                    if (newlayer.renderer == "HeatmapRenderer") {
                       var heatmapRenderer = new HeatmapRenderer();
                       layer.setRenderer(heatmapRenderer);
                    }
                    
                    map.addLayer(layer);
                    
                    if (newlayer.renderer == "ClassedColorRenderer") {
                        layer.on("load", function () {
                             createRenderer(newlayer.field_name);
                         });
                    }
                    
                    if (newlayer.renderer == "ClassedSizeRenderer") {
                        layer.on("load", function () {
                             createSizeRenderer(newlayer.field_name);
                         });
                    }

                     function createRenderer(field) {
                        //smart mapping functionality begins
                        smartMapping.createClassedColorRenderer({
                           layer: layer,
                           field: field,
                           basemap: map.getBasemap(),
                           classificationMethod: "quantile"
                        }).then(function (response) {
                           layer.setRenderer(response.renderer);
                           layer.redraw();
                           //createLegend(map, layer, field);
                        });
                     }
                    
                    function createSizeRenderer(field) {
                         console.log("ClassedSizeRend2");
                        //smart mapping functionality begins
                        smartMapping.createClassedSizeRenderer({
                           layer: layer,
                           field: field,
                           basemap: map.getBasemap(),
                           classificationMethod: "quantile"
                        }).then(function (response) {
                           layer.setRenderer(response.renderer);
                           layer.redraw();
                           //createLegend(map, layer, field);
                        });
                     }
                } 
                else if (newlayer.type == "ImageLayer") {
                         
                    console.log("ArcGISImageServiceLayer " + newlayer.url);
                    var options = {};
                    var swipelayer = false;
                    if (newlayer.options != null) {
                        var imgsvc_options = JSON.parse(newlayer.options);
                        if (imgsvc_options.swipelayer) {
                            swipelayer = imgsvc_options.swipelayer;
                        }
                        if (imgsvc_options.imageServiceParameters) {
                            var params = new ImageServiceParameters();
                            if (imgsvc_options.imageServiceParameters.renderingRule) {
                                var rasterFunction = new RasterFunction(imgsvc_options.imageServiceParameters.renderingRule);
                                if (imgsvc_options.imageServiceParameters.bandIds) {
                                    params.bandIds = imgsvc_options.imageServiceParameters.bandIds;
                                }
                                params.renderingRule = rasterFunction;

                                options.imageServiceParameters = params;
                            }
                        }
                    }
                    var layer = new ArcGISImageServiceLayer(newlayer.url, options);
                    
                    map.addLayer(layer);
                    
                    if (swipelayer) {
                        console.log("Swipe Layer");
                        var swipeWidget = new LayerSwipe({
                            type: "vertical",  //Try switching to "scope" or "horizontal"
                            map: map,
                            layers: [layer]
                        }, this.model.get('_swipe_div'));
                        swipeWidget.startup();
                    }
                }   
                else { // Feature Collection
                
                    console.log(newlayer);

                    var options = { mode: FeatureLayer.MODE_SNAPSHOT };
                    var newlyr_options = {};
                    if (newlayer.options != null) {
                        newlyr_options = Object.assign(options, JSON.parse(newlayer.options));
                    }
                    
                    console.log("***Feature Collection layer###***" );
                    var layer = new FeatureLayer(newlayer, newlyr_options);
                    
                    if (newlayer.options!= null) {
                        var lyr_options = JSON.parse(newlayer.options);
                        
                        
                        console.log("ClassedSizeRend0:"+ lyr_options.renderer);
                        console.log("ClassedSizeRend:" + lyr_options.field_name);
                        
                        if (lyr_options.renderer == "HeatmapRenderer") {
                           var heatmapRenderer = new HeatmapRenderer();
                           var hmoptions = {};
                            
                           if (lyr_options.field_name != null) {
                               hmoptions = {
                                   field: lyr_options.field_name,
                               };
                           }
                           var heatmapRenderer = new HeatmapRenderer(hmoptions);
                           
                           layer.setRenderer(heatmapRenderer);
                        }
                        
                        
                        if (lyr_options.renderer == "ClassedSizeRenderer") {
                            console.log("ClassedSizeRenderer...");
                            setTimeout(function(){ createClassedSizeRenderer(lyr_options.field_name); }, 500);
                            /*layer.on("load", function () {
                                console.log("AAA");
                                 createClassedSizeRenderer(lyr_options.field_name);
                             });*/
                        }
                        
                        if (lyr_options.renderer == "ClassedColorRenderer") {
                            setTimeout(function(){ createClassedColorRenderer(lyr_options.field_name); }, 500);
                            /*layer.on("load", function () {
                                console.log("BBB");
                                 createClassedColorRenderer(lyr_options.field_name);
                             });
                             */
                        }

                         function createClassedColorRenderer(field) {
                            //smart mapping functionality begins
                            smartMapping.createClassedColorRenderer({
                               layer: layer,
                               field: field,
                               basemap: map.getBasemap(),
                               classificationMethod: "quantile"
                            }).then(function (response) {
                               layer.setRenderer(response.renderer);
                               layer.redraw();
                               //createLegend(map, layer, field);
                            });
                         }


                         function createClassedSizeRenderer(field) {
                             console.log("ClassedSizeRend2");
                            //smart mapping functionality begins
                            smartMapping.createClassedSizeRenderer({
                               layer: layer,
                               field: field,
                               basemap: map.getBasemap(),
                               classificationMethod: "quantile"
                            }).then(function (response) {
                               layer.setRenderer(response.renderer);
                               layer.redraw();
                               //createLegend(map, layer, field);
                            });
                         }
                    }
                    
                    
                    map.addLayer(layer);
                }
                
            }
        },
        
        
        center_changed: function() {
            console.log("changing center");
            map.centerAt(this.model.get('center').reverse());
        },
        
        start_time_changed: function() {
            console.log("changing start_time");
            var timeExtent = new TimeExtent();
            timeExtent.startTime = new Date(this.model.get('start_time'));
            timeExtent.endTime = new Date(this.model.get('end_time'));
            map.setTimeExtent(timeExtent);
        },
        
        end_time_changed: function() {
            console.log("changing end_time");
            var timeExtent = new TimeExtent();
            timeExtent.startTime = new Date(this.model.get('start_time'));
            timeExtent.endTime = new Date(this.model.get('end_time'));
            map.setTimeExtent(timeExtent);
        },
        
        basemap_changed: function() {
            map.setBasemap(this.model.get('basemap'));
        },
        
        // Outgoing events to the model
        
        mouse_clicked: function(geometry) { //mapx, mapy) {
            this.send({event: 'mouseclick', message: geometry});//'{ \'x\':' + mapx + ', \'y\':' + mapy + '}'});
        },
        
        draw_end: function(geometry) {
            this.model.set('mode','navigate');
            this.touch();
            this.send({event: 'draw-end', message: geometry});
        },
        /*
        extent_change(extent, zoomed) {
            //console.log(extent);
            //console.log(zoomed);
        },
        */
        
        events: {
            // Dictionary of events and their handlers.
            'click': '_handle_click',
        },
        
        _handle_click: function(){
            /**
             * Handles when the button is clicked.
             */
            this.send({event: 'click', message: 'xyz'});
        },
        
    });

    return {
        MapView : MapView
    };

    //manager.WidgetManager.register_widget_view('MapView', MapView);
});


              /*
              var infoTemplate = new InfoTemplate("${state_name}", "Population (2000):  ${pop2000:NumberFormat}");
              var featureLayer = new FeatureLayer("http://sampleserver6.arcgisonline.com/arcgis/rest/services/USA/MapServer/3",{
                mode: FeatureLayer.MODE_ONDEMAND,
                outFields: ["*"],
                infoTemplate: infoTemplate
              });
              
              //map.addLayer(featureLayer);
              //map.infoWindow.resize(155,75);
              */

